//
// nothingtest.c
//
// Win32 console mode program to fire up requests to the
// NOTHING driver.
//

#include <stdio.h>
#include <windows.h>
#include <winioctl.h>
#include <nothing_ioctl.h>

//
// Simple test application for the RCM project.
//

__cdecl main(ULONG argc, LPSTR *argv)
{
    HANDLE          DriverHandle;
    DWORD           code;
    ULONG           index;
    UCHAR           WriteBuffer[4096];
    UCHAR           ReadBuffer[9096];
    OVERLAPPED      WriteOverlapped;
    OVERLAPPED      ReadOverlapped;
    HANDLE          WriteEvent;
    HANDLE          ReadEvent;
    DWORD           WaitResult;
    DWORD           BytesRead, BytesWritten;
    DWORD           Function;

    //
    // Build the overlapped structures, creating a couple of
    // events to use in the process
    //
    WriteOverlapped.Offset     = 0; 
    WriteOverlapped.OffsetHigh = 0; 
    WriteOverlapped.hEvent     = CreateEvent(NULL, TRUE, FALSE, NULL);

    ReadOverlapped.Offset     = 0; 
    ReadOverlapped.OffsetHigh = 0; 
    ReadOverlapped.hEvent     = CreateEvent(NULL, TRUE, FALSE, NULL);

    //
    // Init the write and read buffers with known data
    //
    memset(ReadBuffer, 0xAA, sizeof(ReadBuffer));

    memset(WriteBuffer, 0xEE, sizeof(WriteBuffer));

    //
    // Open the sample PCI device
    //
    DriverHandle = CreateFile("\\\\.\\NOTHING",  // Name of the NT "device" to open
                            GENERIC_READ|GENERIC_WRITE,  // Access rights requested
                            0,                           // Share access - NONE
                            0,                           // Security attributes - not used!
                            OPEN_EXISTING,               // Device must exist to open it.
                            FILE_FLAG_OVERLAPPED,        // Open for overlapped I/O
                            0);                          // extended attributes - not used!

    //
    // If this call fails, check to figure out what the error is and report it.
    //
    if (DriverHandle == INVALID_HANDLE_VALUE) {

        code = GetLastError();

        printf("CreateFile failed with error 0x%x\n", code);

        return(code);
    }

    //
    // Infinitely print out the list of choices, ask for input, process
    // the request
    //
    while(TRUE)  {

        printf ("\nNOTHING TEST -- Functions:\n\n");
        printf ("\t1. Queue a READ\n");
        printf ("\t2. Issue a WRITE\n");
        printf ("\t3. Wait for READ\n");
        printf ("\t4. Wait for WRITE\n");
        printf ("\t5. Print READ buffer\n");
        printf ("\t6. Print WRITE buffer\n");
        printf ("\t7. Send NOTHING IOCTL\n");
        printf ("\n\t0. Exit\n");
        printf ("\n\tSelection: ");
        scanf ("%x", &Function);

        switch(Function)  {

        case 1:
            //
            // Send a read
            //
            if ( !ReadFile(DriverHandle,
                            ReadBuffer,
                            sizeof(ReadBuffer),
                            &index,
                            &ReadOverlapped)) {

                code = GetLastError();

                if(code != ERROR_IO_PENDING)  {

                    printf("ReadFile failed with error 0x%x\n", code);

                    return(code);
                }
            }
            break;

        case 2:
            //
            // Send a write
            //
            if (!WriteFile(DriverHandle,
                            WriteBuffer,
                            sizeof(WriteBuffer),
                            &index, 
                            &WriteOverlapped)) {

                code = GetLastError();

                if(code != ERROR_IO_PENDING)  {

                    printf("WriteFile failed with error 0x%x\n", code);

                    return(code);
                }
            }
            break;

        case 3:

            //
            // Wait for Read to complete
            //
            if(!GetOverlappedResult(DriverHandle, &ReadOverlapped, &BytesRead, TRUE)) {

                printf("GetOverlappedResult for READ failed?\n"); 

            }
            code = GetLastError();

            printf("Read result = %d., Bytes Read = %d.\n", code, BytesRead);

            break;

        case 4:
            //
            // Wait for WRITE to complete
            //
            if(!GetOverlappedResult(DriverHandle, &WriteOverlapped, &BytesWritten, TRUE))  {

                printf("GetOverlappedResult for WRITE failed?\n"); 

            }

            code = GetLastError();

            printf("Write result = %d., Bytes Written = %d.\n", code, BytesWritten);

            break;

        case 5:
            //
            // Show read buffer
            //
            printf("Printing first 32 bytes of READ buffer\n");

            for(index = 0; index < 32; index++)  {

                printf("0x%0x,",ReadBuffer[index]);
            }

            break;

        case 6:
            //
            // Show WRITE buffer
            //
            printf("Printing first 32 bytes of WRITE buffer\n");

            for(index = 0; index < 32; index++)  {

                printf("0x%0x,",WriteBuffer[index]);
            }

            break;

        case 7:
            //
            // Test the IOCTL interface
            //
            if (!DeviceIoControl(DriverHandle,
                                (DWORD)IOCTL_OSR_NOTHING,
                                0,      // Ptr to InBuffer
                                0,      // Length of InBuffer
                                0,      // Ptr to OutBuffer
                                0,      // Length of OutBuffer
                                &index, // BytesReturned
                                0) )    // Ptr to Overlapped structure
                                {

                code = GetLastError();

                printf("DeviceIoControl (reset) failed with error 0x%x\n", code);

                return(code);
            }

            printf("DeviceIoControl worked OK. \n");

            break;

        case 0:

            //
            // zero is get out!
            //
            return(0);

        }
    }   
}
